package com.ashokit.ticketbooking.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ashokit.ticketbooking.model.Ticket;
import com.ashokit.ticketbooking.service.TicketService;

@RestController
@RequestMapping(value="/tickets")

//http://locahost:8080/ticket ==> TicketController obj=ioc.getTicketController();
public class TicketController {
	
	/*
	 * 
	 * Controller ==> Entry Point for all external applications
	 * RestController => Entry Point for all external application which are interacting with REST Protocol
	 * 
	 * In Spring boot ==> JAX RS ==> Rest Services API by default availble and developer no need to write any boiler
	 * plate code.
	 * 
	 * Why RestService ==> Heterogenious Communication ==> SOAP Web Services ==> Difficult to implement,Schema, XSD, proxy 
	 *                    Rest Protocol ==> Simple to implement
	 *                                  ==> Runs on HTTP Protocol
	 *                                  ==> Supports with JSON and XML
	 *     
	 *     Both SOAP and REST are part of webservice
	 *     
	 *     Spring boot automated REST API
	 *     
	 *       @GetMapping ==> @Get + @RequestMapping ==> Retriving the data
	 *       @PostMapping ==> @Post+ @RequestMapping ==> Creating the data
	 *       @PutMapping ==> @Put+@RequestMapping ==> Updates the data
	 *       @DeleteMapping ==> @Delete+@RequestMapping ==> Deletes the data
	 *       
	 *       
	 *       Also Spring boot comes with JAXB Framework which JackSon LIbrary which will convert
	 *                   JAVA ==> JSON (Marshalling)
	 *                   JSON ==> JAVA (UnMarshalling)
	 *                   
	 *                   
	 *       Heterogenious communication happens through JSON ==> By default Spring boot converts java to JSON and JSON to Java
	 *                      It internally uses Jackson Library ==> Marshalling and Unmarshalling
	 *                      
	 *                         Marshalling ==> Java to JSON
	 *                         UnMarshalling ==> Json to java
	 *                         
	 *        URL Uniqueness ==> @RestController Path + @RequestMapping Path + @VERB
	 *         in case of get ==> http://localhost:8080/tickets/1 + Get 
	 *         in case of delete ==> http://localhost:8080/tickets/1 + Delete
	 */
	
	
	@Autowired
	private TicketService ticketService;
	
	
	
	//Get All Tickets
	
	@GetMapping(value="/all") //http://localhost:8080/ticket/all 
	//http://localhost:8080/ticket/all ==> obj.getAllTickets();
	public Iterable<Ticket> getAllTickets(){
		return ticketService.getAllTickets();
	}
	
	//Create Ticket
	
	
	@PostMapping(value="/create") //http://localhost:8080/ticket/create
	//http://localhost:8080/create ==> obj.createTicket();
	//@RequestBody ==> Every Request will have body section ==> you should have json in the body section
	public Ticket createTicket(@RequestBody Ticket inputObj) {
		return ticketService.createTicket(inputObj);
	}
	
	//Get Ticket
	//http://localhost:8080/tickets/ticket/1 ==> 1st Ticket
	//http://localhost:8080/tickets/ticket/2 ==> 2nd Ticket
	//http://localhost:8080/ticket/ticket/100 ==> 100th Ticket
	//{ } ==> Dynamic Variable
	@GetMapping(value="/ticket/{ticketId}")
	public Ticket getTicket(@PathVariable("ticketId") Integer ticketId) {
		return ticketService.getTicket(ticketId);
	}
	
	
	
	//Update Ticket
	//http://localhost:8080/tickets/1/a@b.com
	//http://localhost:8080/tickets/2/b@c.com
	@PutMapping(value="/{ticketId}/{newEmail}")
	public Ticket updateTicket(@PathVariable("ticketId") Integer ticketId,
			@PathVariable("newEmail") String newEmail) {
		return ticketService.updateTicket(ticketId, newEmail);
	}
	

	
	//http://localhost:8080/tickets/1 ==> Delete 1st ticket
	//http://localhost:8080/tickets/2 ==> Delete 2nd Ticket
	@DeleteMapping(value="/{ticketId}")
	public void deleteTicket(@PathVariable("ticketId") Integer ticketId) {
		ticketService.deleteTicket(ticketId);
	}
	
	
	
	
	
	
	
	
	//Delete Ticket

}
